//**********************************************************************`
//* This is an include file generated by Message Compiler.             *`
//*                                                                    *`
//* Copyright (c) Microsoft Corporation. All Rights Reserved.          *`
//**********************************************************************`
#pragma once
#include <wmistr.h>
#include <evntrace.h>
#include "evntprov.h"
//
//  Initial Defs
//
#if !defined(ETW_INLINE)
#define ETW_INLINE DECLSPEC_NOINLINE __inline
#endif

#if defined(__cplusplus)
extern "C" {
#endif

//
// Allow Diasabling of code generation
//
#ifndef MCGEN_DISABLE_PROVIDER_CODE_GENERATION
#if  !defined(McGenDebug)
#define McGenDebug(a,b)
#endif 


#if !defined(MCGEN_TRACE_CONTEXT_DEF)
#define MCGEN_TRACE_CONTEXT_DEF
typedef struct _MCGEN_TRACE_CONTEXT
{
    TRACEHANDLE            RegistrationHandle;
    TRACEHANDLE            Logger;
    ULONGLONG              MatchAnyKeyword;
    ULONGLONG              MatchAllKeyword;
    ULONG                  Flags;
    ULONG                  IsEnabled;
    UCHAR                  Level; 
    UCHAR                  Reserve;
    USHORT                 EnableBitsCount;
    PULONG                 EnableBitMask;
    const ULONGLONG*       EnableKeyWords;
    const UCHAR*           EnableLevel;
} MCGEN_TRACE_CONTEXT, *PMCGEN_TRACE_CONTEXT;
#endif

#if !defined(MCGEN_LEVEL_KEYWORD_ENABLED_DEF)
#define MCGEN_LEVEL_KEYWORD_ENABLED_DEF
FORCEINLINE
BOOLEAN
McGenLevelKeywordEnabled(
    _In_ PMCGEN_TRACE_CONTEXT EnableInfo,
    _In_ UCHAR Level,
    _In_ ULONGLONG Keyword
    )
{
    //
    // Check if the event Level is lower than the level at which
    // the channel is enabled.
    // If the event Level is 0 or the channel is enabled at level 0,
    // all levels are enabled.
    //

    if ((Level <= EnableInfo->Level) || // This also covers the case of Level == 0.
        (EnableInfo->Level == 0)) {

        //
        // Check if Keyword is enabled
        //

        if ((Keyword == (ULONGLONG)0) ||
            ((Keyword & EnableInfo->MatchAnyKeyword) &&
             ((Keyword & EnableInfo->MatchAllKeyword) == EnableInfo->MatchAllKeyword))) {
            return TRUE;
        }
    }

    return FALSE;

}
#endif

#if !defined(MCGEN_EVENT_ENABLED_DEF)
#define MCGEN_EVENT_ENABLED_DEF
FORCEINLINE
BOOLEAN
McGenEventEnabled(
    _In_ PMCGEN_TRACE_CONTEXT EnableInfo,
    _In_ PCEVENT_DESCRIPTOR EventDescriptor
    )
{

    return McGenLevelKeywordEnabled(EnableInfo, EventDescriptor->Level, EventDescriptor->Keyword);

}
#endif


//
// EnableCheckMacro
//
#ifndef MCGEN_ENABLE_CHECK
#define MCGEN_ENABLE_CHECK(Context, Descriptor) (Context.IsEnabled &&  McGenEventEnabled(&Context, &Descriptor))
#endif

#if !defined(MCGEN_CONTROL_CALLBACK)
#define MCGEN_CONTROL_CALLBACK

DECLSPEC_NOINLINE __inline
VOID
__stdcall
McGenControlCallbackV2(
    _In_ LPCGUID SourceId,
    _In_ ULONG ControlCode,
    _In_ UCHAR Level,
    _In_ ULONGLONG MatchAnyKeyword,
    _In_ ULONGLONG MatchAllKeyword,
    _In_opt_ PEVENT_FILTER_DESCRIPTOR FilterData,
    _Inout_opt_ PVOID CallbackContext
    )
/*++

Routine Description:

    This is the notification callback for Vista.

Arguments:

    SourceId - The GUID that identifies the session that enabled the provider. 

    ControlCode - The parameter indicates whether the provider 
                  is being enabled or disabled.

    Level - The level at which the event is enabled.

    MatchAnyKeyword - The bitmask of keywords that the provider uses to 
                      determine the category of events that it writes.

    MatchAllKeyword - This bitmask additionally restricts the category 
                      of events that the provider writes. 

    FilterData - The provider-defined data.

    CallbackContext - The context of the callback that is defined when the provider 
                      called EtwRegister to register itself.

Remarks:

    ETW calls this function to notify provider of enable/disable

--*/
{
    PMCGEN_TRACE_CONTEXT Ctx = (PMCGEN_TRACE_CONTEXT)CallbackContext;
    ULONG Ix;
#ifndef MCGEN_PRIVATE_ENABLE_CALLBACK_V2
    UNREFERENCED_PARAMETER(SourceId);
    UNREFERENCED_PARAMETER(FilterData);
#endif

    if (Ctx == NULL) {
        return;
    }

    switch (ControlCode) {

        case EVENT_CONTROL_CODE_ENABLE_PROVIDER:
            Ctx->Level = Level;
            Ctx->MatchAnyKeyword = MatchAnyKeyword;
            Ctx->MatchAllKeyword = MatchAllKeyword;
            Ctx->IsEnabled = EVENT_CONTROL_CODE_ENABLE_PROVIDER;

            for (Ix = 0; Ix < Ctx->EnableBitsCount; Ix += 1) {
                if (McGenLevelKeywordEnabled(Ctx, Ctx->EnableLevel[Ix], Ctx->EnableKeyWords[Ix]) != FALSE) {
                    Ctx->EnableBitMask[Ix >> 5] |= (1 << (Ix % 32));
                } else {
                    Ctx->EnableBitMask[Ix >> 5] &= ~(1 << (Ix % 32));
                }
            }
            break;

        case EVENT_CONTROL_CODE_DISABLE_PROVIDER:
            Ctx->IsEnabled = EVENT_CONTROL_CODE_DISABLE_PROVIDER;
            Ctx->Level = 0;
            Ctx->MatchAnyKeyword = 0;
            Ctx->MatchAllKeyword = 0;
            if (Ctx->EnableBitsCount > 0) {
                RtlZeroMemory(Ctx->EnableBitMask, (((Ctx->EnableBitsCount - 1) / 32) + 1) * sizeof(ULONG));
            }
            break;
 
        default:
            break;
    }

#ifdef MCGEN_PRIVATE_ENABLE_CALLBACK_V2
    //
    // Call user defined callback
    //
    MCGEN_PRIVATE_ENABLE_CALLBACK_V2(
        SourceId,
        ControlCode,
        Level,
        MatchAnyKeyword,
        MatchAllKeyword,
        FilterData,
        CallbackContext
        );
#endif
   
    return;
}

#endif
#endif // MCGEN_DISABLE_PROVIDER_CODE_GENERATION
//+
// Provider Multi-Main Event Count 8
//+
EXTERN_C __declspec(selectany) const GUID MULTI_MAIN = {0x231cf54b, 0x22a0, 0x49e4, {0xa5, 0x9a, 0x47, 0x05, 0x2a, 0x30, 0xff, 0xed}};

//
// Opcodes
//
#define _BeginOpcode 0xa
#define _EndOpcode 0xb
#define _StepOpcode 0xc
#define _MarkOpcode 0xd
#define _InformationOpcode 0xe

//
// Tasks
//
#define Block_Task 0x1
EXTERN_C __declspec(selectany) const GUID BlockId = {0x4e9a75eb, 0x4fba, 0x4ba0, {0x9a, 0x1b, 0x21, 0x75, 0xb6, 0x71, 0xa1, 0x6d}};
#define ThreadID_Task 0x2
EXTERN_C __declspec(selectany) const GUID ThreadIDId = {0xf2ec684f, 0xad3a, 0x4af5, {0x8b, 0x10, 0xe7, 0xb2, 0x9a, 0xf6, 0x7e, 0xe2}};
//
// Keyword
//
#define MULTI_MAIN_KEYWORD_HighFrequency 0x2
#define MULTI_MAIN_KEYWORD_NormalFrequency 0x1

//
// Event Descriptors
//
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Start = {0x64, 0x0, 0x0, 0x0, 0xa, 0x1, 0x1};
#define Start_value 0x64
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Stop = {0x65, 0x0, 0x0, 0x0, 0xb, 0x1, 0x1};
#define Stop_value 0x65
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Mark = {0x66, 0x0, 0x0, 0x0, 0xd, 0x1, 0x1};
#define Mark_value 0x66
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Thread_ID = {0x67, 0x0, 0x0, 0x0, 0xe, 0x2, 0x1};
#define Thread_ID_value 0x67
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Mark1I = {0x68, 0x0, 0x0, 0x0, 0xd, 0x1, 0x1};
#define Mark1I_value 0x68
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Mark2I = {0x69, 0x0, 0x0, 0x0, 0xd, 0x1, 0x1};
#define Mark2I_value 0x69
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Mark1F = {0x6a, 0x0, 0x0, 0x0, 0xd, 0x1, 0x1};
#define Mark1F_value 0x6a
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Mark2F = {0x6b, 0x0, 0x0, 0x0, 0xd, 0x1, 0x1};
#define Mark2F_value 0x6b

//
// Note on Generate Code from Manifest Windows Vista and above
//
//Structures :  are handled as a size and pointer pairs. The macro for the event will have an extra 
//parameter for the size in bytes of the structure. Make sure that your structures have no extra padding.
//
//Strings: There are several cases that can be described in the manifest. For array of variable length 
//strings, the generated code will take the count of characters for the whole array as an input parameter. 
//
//SID No support for array of SIDs, the macro will take a pointer to the SID and use appropriate 
//GetLengthSid function to get the length.
//

//
// Allow Diasabling of code generation
//
#ifndef MCGEN_DISABLE_PROVIDER_CODE_GENERATION

//
// Globals 
//


//
// Event Enablement Bits
//

EXTERN_C __declspec(selectany) DECLSPEC_CACHEALIGN ULONG Multi_MainEnableBits[1];
EXTERN_C __declspec(selectany) const ULONGLONG Multi_MainKeywords[1] = {0x1};
EXTERN_C __declspec(selectany) const UCHAR Multi_MainLevels[1] = {0};
EXTERN_C __declspec(selectany) MCGEN_TRACE_CONTEXT MULTI_MAIN_Context = {0, 0, 0, 0, 0, 0, 0, 0, 1, Multi_MainEnableBits, Multi_MainKeywords, Multi_MainLevels};

EXTERN_C __declspec(selectany) REGHANDLE Multi_MainHandle = (REGHANDLE)0;

#if !defined(McGenEventRegisterUnregister)
#define McGenEventRegisterUnregister
DECLSPEC_NOINLINE __inline
ULONG __stdcall
McGenEventRegister(
    _In_ LPCGUID ProviderId,
    _In_opt_ PENABLECALLBACK EnableCallback,
    _In_opt_ PVOID CallbackContext,
    _Inout_ PREGHANDLE RegHandle
    )
/*++

Routine Description:

    This function register the provider with ETW USER mode.

Arguments:
    ProviderId - Provider Id to be register with ETW.

    EnableCallback - Callback to be used.

    CallbackContext - Context for this provider.

    RegHandle - Pointer to Registration handle.

Remarks:

    If the handle != NULL will return ERROR_SUCCESS

--*/
{
    ULONG Error;


    if (*RegHandle) {
        //
        // already registered
        //
        return ERROR_SUCCESS;
    }

    Error = EventRegister( ProviderId, EnableCallback, CallbackContext, RegHandle); 

    return Error;
}


DECLSPEC_NOINLINE __inline
ULONG __stdcall
McGenEventUnregister(_Inout_ PREGHANDLE RegHandle)
/*++

Routine Description:

    Unregister from ETW USER mode

Arguments:
            RegHandle this is the pointer to the provider context
Remarks:
            If Provider has not register RegHandle = NULL,
            return ERROR_SUCCESS
--*/
{
    ULONG Error;


    if(!(*RegHandle)) {
        //
        // Provider has not registerd
        //
        return ERROR_SUCCESS;
    }

    Error = EventUnregister(*RegHandle); 
    *RegHandle = (REGHANDLE)0;
    
    return Error;
}
#endif
//
// Register with ETW Vista +
//
#ifndef EventRegisterMulti_Main
#define EventRegisterMulti_Main() McGenEventRegister(&MULTI_MAIN, McGenControlCallbackV2, &MULTI_MAIN_Context, &Multi_MainHandle) 
#endif

//
// UnRegister with ETW
//
#ifndef EventUnregisterMulti_Main
#define EventUnregisterMulti_Main() McGenEventUnregister(&Multi_MainHandle) 
#endif

//
// Enablement check macro for Start
//

#define EventEnabledStart() ((Multi_MainEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Start
//
#define EventWriteStart(Description, Depth)\
        EventEnabledStart() ?\
        Template_sd(Multi_MainHandle, &Start, Description, Depth)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Stop
//

#define EventEnabledStop() ((Multi_MainEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Stop
//
#define EventWriteStop(Description, Depth, Duration__ms_)\
        EventEnabledStop() ?\
        Template_sdf(Multi_MainHandle, &Stop, Description, Depth, Duration__ms_)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Mark
//

#define EventEnabledMark() ((Multi_MainEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Mark
//
#define EventWriteMark(Description)\
        EventEnabledMark() ?\
        Template_s(Multi_MainHandle, &Mark, Description)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Thread_ID
//

#define EventEnabledThread_ID() ((Multi_MainEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Thread_ID
//
#define EventWriteThread_ID(ThreadID, ThreadName)\
        EventEnabledThread_ID() ?\
        Template_ds(Multi_MainHandle, &Thread_ID, ThreadID, ThreadName)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Mark1I
//

#define EventEnabledMark1I() ((Multi_MainEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Mark1I
//
#define EventWriteMark1I(Description, Data1)\
        EventEnabledMark1I() ?\
        Template_sd(Multi_MainHandle, &Mark1I, Description, Data1)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Mark2I
//

#define EventEnabledMark2I() ((Multi_MainEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Mark2I
//
#define EventWriteMark2I(Description, Data1, Data2)\
        EventEnabledMark2I() ?\
        Template_sdd(Multi_MainHandle, &Mark2I, Description, Data1, Data2)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Mark1F
//

#define EventEnabledMark1F() ((Multi_MainEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Mark1F
//
#define EventWriteMark1F(Description, Data1)\
        EventEnabledMark1F() ?\
        Template_sf(Multi_MainHandle, &Mark1F, Description, Data1)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Mark2F
//

#define EventEnabledMark2F() ((Multi_MainEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Mark2F
//
#define EventWriteMark2F(Description, Data1, Data2)\
        EventEnabledMark2F() ?\
        Template_sff(Multi_MainHandle, &Mark2F, Description, Data1, Data2)\
        : ERROR_SUCCESS\

#endif // MCGEN_DISABLE_PROVIDER_CODE_GENERATION

//+
// Provider Multi-Worker Event Count 3
//+
EXTERN_C __declspec(selectany) const GUID MULTI_WORKER = {0xe9c3da11, 0xe2a5, 0x48fd, {0x9c, 0xd3, 0x17, 0xe7, 0xc7, 0x64, 0xd3, 0x03}};

//
// Opcodes
//
#define _BeginOpcode 0xa
#define _EndOpcode 0xb
#define _StepOpcode 0xc
#define _MarkOpcode 0xd

//
// Tasks
//
#define Block_Task 0x1
EXTERN_C __declspec(selectany) const GUID BlockWorkerId = {0x9e4a04f0, 0xc229, 0x4a16, {0x94, 0x54, 0xd1, 0x2a, 0x5b, 0xa2, 0x41, 0x84}};
//
// Keyword
//
#define MULTI_WORKER_KEYWORD_HighFrequency 0x2
#define MULTI_WORKER_KEYWORD_NormalFrequency 0x1

//
// Event Descriptors
//
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR StartWorker = {0x64, 0x0, 0x0, 0x0, 0xa, 0x1, 0x0};
#define StartWorker_value 0x64
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR StopWorker = {0x65, 0x0, 0x0, 0x0, 0xb, 0x1, 0x0};
#define StopWorker_value 0x65
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR MarkWorker = {0x66, 0x0, 0x0, 0x0, 0xd, 0x1, 0x0};
#define MarkWorker_value 0x66

//
// Note on Generate Code from Manifest Windows Vista and above
//
//Structures :  are handled as a size and pointer pairs. The macro for the event will have an extra 
//parameter for the size in bytes of the structure. Make sure that your structures have no extra padding.
//
//Strings: There are several cases that can be described in the manifest. For array of variable length 
//strings, the generated code will take the count of characters for the whole array as an input parameter. 
//
//SID No support for array of SIDs, the macro will take a pointer to the SID and use appropriate 
//GetLengthSid function to get the length.
//

//
// Allow Diasabling of code generation
//
#ifndef MCGEN_DISABLE_PROVIDER_CODE_GENERATION

//
// Globals 
//


//
// Event Enablement Bits
//

EXTERN_C __declspec(selectany) DECLSPEC_CACHEALIGN ULONG Multi_WorkerEnableBits[1];
EXTERN_C __declspec(selectany) const ULONGLONG Multi_WorkerKeywords[1] = {0x0};
EXTERN_C __declspec(selectany) const UCHAR Multi_WorkerLevels[1] = {0};
EXTERN_C __declspec(selectany) MCGEN_TRACE_CONTEXT MULTI_WORKER_Context = {0, 0, 0, 0, 0, 0, 0, 0, 1, Multi_WorkerEnableBits, Multi_WorkerKeywords, Multi_WorkerLevels};

EXTERN_C __declspec(selectany) REGHANDLE Multi_WorkerHandle = (REGHANDLE)0;

#if !defined(McGenEventRegisterUnregister)
#define McGenEventRegisterUnregister
DECLSPEC_NOINLINE __inline
ULONG __stdcall
McGenEventRegister(
    _In_ LPCGUID ProviderId,
    _In_opt_ PENABLECALLBACK EnableCallback,
    _In_opt_ PVOID CallbackContext,
    _Inout_ PREGHANDLE RegHandle
    )
/*++

Routine Description:

    This function register the provider with ETW USER mode.

Arguments:
    ProviderId - Provider Id to be register with ETW.

    EnableCallback - Callback to be used.

    CallbackContext - Context for this provider.

    RegHandle - Pointer to Registration handle.

Remarks:

    If the handle != NULL will return ERROR_SUCCESS

--*/
{
    ULONG Error;


    if (*RegHandle) {
        //
        // already registered
        //
        return ERROR_SUCCESS;
    }

    Error = EventRegister( ProviderId, EnableCallback, CallbackContext, RegHandle); 

    return Error;
}


DECLSPEC_NOINLINE __inline
ULONG __stdcall
McGenEventUnregister(_Inout_ PREGHANDLE RegHandle)
/*++

Routine Description:

    Unregister from ETW USER mode

Arguments:
            RegHandle this is the pointer to the provider context
Remarks:
            If Provider has not register RegHandle = NULL,
            return ERROR_SUCCESS
--*/
{
    ULONG Error;


    if(!(*RegHandle)) {
        //
        // Provider has not registerd
        //
        return ERROR_SUCCESS;
    }

    Error = EventUnregister(*RegHandle); 
    *RegHandle = (REGHANDLE)0;
    
    return Error;
}
#endif
//
// Register with ETW Vista +
//
#ifndef EventRegisterMulti_Worker
#define EventRegisterMulti_Worker() McGenEventRegister(&MULTI_WORKER, McGenControlCallbackV2, &MULTI_WORKER_Context, &Multi_WorkerHandle) 
#endif

//
// UnRegister with ETW
//
#ifndef EventUnregisterMulti_Worker
#define EventUnregisterMulti_Worker() McGenEventUnregister(&Multi_WorkerHandle) 
#endif

//
// Enablement check macro for StartWorker
//

#define EventEnabledStartWorker() ((Multi_WorkerEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for StartWorker
//
#define EventWriteStartWorker(Description, Depth)\
        EventEnabledStartWorker() ?\
        Template_sd(Multi_WorkerHandle, &StartWorker, Description, Depth)\
        : ERROR_SUCCESS\

//
// Enablement check macro for StopWorker
//

#define EventEnabledStopWorker() ((Multi_WorkerEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for StopWorker
//
#define EventWriteStopWorker(Description, Depth, Duration__ms_)\
        EventEnabledStopWorker() ?\
        Template_sdf(Multi_WorkerHandle, &StopWorker, Description, Depth, Duration__ms_)\
        : ERROR_SUCCESS\

//
// Enablement check macro for MarkWorker
//

#define EventEnabledMarkWorker() ((Multi_WorkerEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for MarkWorker
//
#define EventWriteMarkWorker(Description)\
        EventEnabledMarkWorker() ?\
        Template_s(Multi_WorkerHandle, &MarkWorker, Description)\
        : ERROR_SUCCESS\

#endif // MCGEN_DISABLE_PROVIDER_CODE_GENERATION

//+
// Provider Multi-FrameRate Event Count 1
//+
EXTERN_C __declspec(selectany) const GUID MULTI_FRAMERATE = {0x2b25961d, 0xba6e, 0x4d79, {0xbe, 0xc7, 0x36, 0x05, 0x36, 0x6e, 0x2e, 0x09}};

//
// Opcodes
//
#define _RenderFrameMarkOpcode 0xa

//
// Tasks
//
#define Frame_Task 0x1
EXTERN_C __declspec(selectany) const GUID FrameId = {0x43dada85, 0x49b6, 0x4438, {0x83, 0xd6, 0x93, 0x14, 0x77, 0x63, 0x5d, 0xe3}};
//
// Keyword
//
#define MULTI_FRAMERATE_KEYWORD_HighFrequency 0x2
#define MULTI_FRAMERATE_KEYWORD_NormalFrequency 0x1

//
// Event Descriptors
//
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR RenderFrameMark = {0xc8, 0x0, 0x0, 0x0, 0xa, 0x1, 0x1};
#define RenderFrameMark_value 0xc8

//
// Note on Generate Code from Manifest Windows Vista and above
//
//Structures :  are handled as a size and pointer pairs. The macro for the event will have an extra 
//parameter for the size in bytes of the structure. Make sure that your structures have no extra padding.
//
//Strings: There are several cases that can be described in the manifest. For array of variable length 
//strings, the generated code will take the count of characters for the whole array as an input parameter. 
//
//SID No support for array of SIDs, the macro will take a pointer to the SID and use appropriate 
//GetLengthSid function to get the length.
//

//
// Allow Diasabling of code generation
//
#ifndef MCGEN_DISABLE_PROVIDER_CODE_GENERATION

//
// Globals 
//


//
// Event Enablement Bits
//

EXTERN_C __declspec(selectany) DECLSPEC_CACHEALIGN ULONG Multi_FrameRateEnableBits[1];
EXTERN_C __declspec(selectany) const ULONGLONG Multi_FrameRateKeywords[1] = {0x1};
EXTERN_C __declspec(selectany) const UCHAR Multi_FrameRateLevels[1] = {0};
EXTERN_C __declspec(selectany) MCGEN_TRACE_CONTEXT MULTI_FRAMERATE_Context = {0, 0, 0, 0, 0, 0, 0, 0, 1, Multi_FrameRateEnableBits, Multi_FrameRateKeywords, Multi_FrameRateLevels};

EXTERN_C __declspec(selectany) REGHANDLE Multi_FrameRateHandle = (REGHANDLE)0;

#if !defined(McGenEventRegisterUnregister)
#define McGenEventRegisterUnregister
DECLSPEC_NOINLINE __inline
ULONG __stdcall
McGenEventRegister(
    _In_ LPCGUID ProviderId,
    _In_opt_ PENABLECALLBACK EnableCallback,
    _In_opt_ PVOID CallbackContext,
    _Inout_ PREGHANDLE RegHandle
    )
/*++

Routine Description:

    This function register the provider with ETW USER mode.

Arguments:
    ProviderId - Provider Id to be register with ETW.

    EnableCallback - Callback to be used.

    CallbackContext - Context for this provider.

    RegHandle - Pointer to Registration handle.

Remarks:

    If the handle != NULL will return ERROR_SUCCESS

--*/
{
    ULONG Error;


    if (*RegHandle) {
        //
        // already registered
        //
        return ERROR_SUCCESS;
    }

    Error = EventRegister( ProviderId, EnableCallback, CallbackContext, RegHandle); 

    return Error;
}


DECLSPEC_NOINLINE __inline
ULONG __stdcall
McGenEventUnregister(_Inout_ PREGHANDLE RegHandle)
/*++

Routine Description:

    Unregister from ETW USER mode

Arguments:
            RegHandle this is the pointer to the provider context
Remarks:
            If Provider has not register RegHandle = NULL,
            return ERROR_SUCCESS
--*/
{
    ULONG Error;


    if(!(*RegHandle)) {
        //
        // Provider has not registerd
        //
        return ERROR_SUCCESS;
    }

    Error = EventUnregister(*RegHandle); 
    *RegHandle = (REGHANDLE)0;
    
    return Error;
}
#endif
//
// Register with ETW Vista +
//
#ifndef EventRegisterMulti_FrameRate
#define EventRegisterMulti_FrameRate() McGenEventRegister(&MULTI_FRAMERATE, McGenControlCallbackV2, &MULTI_FRAMERATE_Context, &Multi_FrameRateHandle) 
#endif

//
// UnRegister with ETW
//
#ifndef EventUnregisterMulti_FrameRate
#define EventUnregisterMulti_FrameRate() McGenEventUnregister(&Multi_FrameRateHandle) 
#endif

//
// Enablement check macro for RenderFrameMark
//

#define EventEnabledRenderFrameMark() ((Multi_FrameRateEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for RenderFrameMark
//
#define EventWriteRenderFrameMark(Frame_number, Duration__ms_)\
        EventEnabledRenderFrameMark() ?\
        Template_df(Multi_FrameRateHandle, &RenderFrameMark, Frame_number, Duration__ms_)\
        : ERROR_SUCCESS\

#endif // MCGEN_DISABLE_PROVIDER_CODE_GENERATION

//+
// Provider Multi-Input Event Count 5
//+
EXTERN_C __declspec(selectany) const GUID MULTI_INPUT = {0x70e2503b, 0xc6f3, 0x4780, {0xb3, 0x23, 0xbd, 0x8e, 0xd0, 0xc6, 0x1b, 0xf8}};

//
// Opcodes
//
#define _MouseDownOpcode 0xa
#define _MouseUpOpcode 0xb
#define _MouseMoveOpcode 0xc
#define _MouseWheelOpcode 0xd
#define _KeyDownOpcode 0xe

//
// Tasks
//
#define Mouse_Task 0x1
EXTERN_C __declspec(selectany) const GUID MouseId = {0x8317ff5b, 0x2854, 0x4ff4, {0xaf, 0x5f, 0xf4, 0xa1, 0x23, 0xb5, 0x14, 0xae}};
#define Keyboard_Task 0x2
EXTERN_C __declspec(selectany) const GUID KeyboardId = {0x8a86f981, 0xc58b, 0x4940, {0x85, 0xe0, 0xc0, 0x3b, 0x61, 0x2f, 0xfb, 0xe8}};
//
// Keyword
//
#define MULTI_INPUT_KEYWORD_HighFrequency 0x2
#define MULTI_INPUT_KEYWORD_NormalFrequency 0x1

//
// Event Descriptors
//
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Mouse_down = {0x190, 0x0, 0x0, 0x0, 0xa, 0x1, 0x1};
#define Mouse_down_value 0x190
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Mouse_up = {0x191, 0x0, 0x0, 0x0, 0xb, 0x1, 0x1};
#define Mouse_up_value 0x191
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Mouse_move = {0x192, 0x0, 0x0, 0x0, 0xc, 0x1, 0x2};
#define Mouse_move_value 0x192
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Mouse_wheel = {0x193, 0x0, 0x0, 0x0, 0xd, 0x1, 0x1};
#define Mouse_wheel_value 0x193
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR Key_down = {0x194, 0x0, 0x0, 0x0, 0xe, 0x2, 0x1};
#define Key_down_value 0x194

//
// Note on Generate Code from Manifest Windows Vista and above
//
//Structures :  are handled as a size and pointer pairs. The macro for the event will have an extra 
//parameter for the size in bytes of the structure. Make sure that your structures have no extra padding.
//
//Strings: There are several cases that can be described in the manifest. For array of variable length 
//strings, the generated code will take the count of characters for the whole array as an input parameter. 
//
//SID No support for array of SIDs, the macro will take a pointer to the SID and use appropriate 
//GetLengthSid function to get the length.
//

//
// Allow Diasabling of code generation
//
#ifndef MCGEN_DISABLE_PROVIDER_CODE_GENERATION

//
// Globals 
//


//
// Event Enablement Bits
//

EXTERN_C __declspec(selectany) DECLSPEC_CACHEALIGN ULONG Multi_InputEnableBits[1];
EXTERN_C __declspec(selectany) const ULONGLONG Multi_InputKeywords[2] = {0x1, 0x2};
EXTERN_C __declspec(selectany) const UCHAR Multi_InputLevels[2] = {0, 0};
EXTERN_C __declspec(selectany) MCGEN_TRACE_CONTEXT MULTI_INPUT_Context = {0, 0, 0, 0, 0, 0, 0, 0, 2, Multi_InputEnableBits, Multi_InputKeywords, Multi_InputLevels};

EXTERN_C __declspec(selectany) REGHANDLE Multi_InputHandle = (REGHANDLE)0;

#if !defined(McGenEventRegisterUnregister)
#define McGenEventRegisterUnregister
DECLSPEC_NOINLINE __inline
ULONG __stdcall
McGenEventRegister(
    _In_ LPCGUID ProviderId,
    _In_opt_ PENABLECALLBACK EnableCallback,
    _In_opt_ PVOID CallbackContext,
    _Inout_ PREGHANDLE RegHandle
    )
/*++

Routine Description:

    This function register the provider with ETW USER mode.

Arguments:
    ProviderId - Provider Id to be register with ETW.

    EnableCallback - Callback to be used.

    CallbackContext - Context for this provider.

    RegHandle - Pointer to Registration handle.

Remarks:

    If the handle != NULL will return ERROR_SUCCESS

--*/
{
    ULONG Error;


    if (*RegHandle) {
        //
        // already registered
        //
        return ERROR_SUCCESS;
    }

    Error = EventRegister( ProviderId, EnableCallback, CallbackContext, RegHandle); 

    return Error;
}


DECLSPEC_NOINLINE __inline
ULONG __stdcall
McGenEventUnregister(_Inout_ PREGHANDLE RegHandle)
/*++

Routine Description:

    Unregister from ETW USER mode

Arguments:
            RegHandle this is the pointer to the provider context
Remarks:
            If Provider has not register RegHandle = NULL,
            return ERROR_SUCCESS
--*/
{
    ULONG Error;


    if(!(*RegHandle)) {
        //
        // Provider has not registerd
        //
        return ERROR_SUCCESS;
    }

    Error = EventUnregister(*RegHandle); 
    *RegHandle = (REGHANDLE)0;
    
    return Error;
}
#endif
//
// Register with ETW Vista +
//
#ifndef EventRegisterMulti_Input
#define EventRegisterMulti_Input() McGenEventRegister(&MULTI_INPUT, McGenControlCallbackV2, &MULTI_INPUT_Context, &Multi_InputHandle) 
#endif

//
// UnRegister with ETW
//
#ifndef EventUnregisterMulti_Input
#define EventUnregisterMulti_Input() McGenEventUnregister(&Multi_InputHandle) 
#endif

//
// Enablement check macro for Mouse_down
//

#define EventEnabledMouse_down() ((Multi_InputEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Mouse_down
//
#define EventWriteMouse_down(Button_Type, Flags, x, y)\
        EventEnabledMouse_down() ?\
        Template_dqdd(Multi_InputHandle, &Mouse_down, Button_Type, Flags, x, y)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Mouse_up
//

#define EventEnabledMouse_up() ((Multi_InputEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Mouse_up
//
#define EventWriteMouse_up(Button_Type, Flags, x, y)\
        EventEnabledMouse_up() ?\
        Template_dqdd(Multi_InputHandle, &Mouse_up, Button_Type, Flags, x, y)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Mouse_move
//

#define EventEnabledMouse_move() ((Multi_InputEnableBits[0] & 0x00000002) != 0)

//
// Event Macro for Mouse_move
//
#define EventWriteMouse_move(Flags, x, y)\
        EventEnabledMouse_move() ?\
        Template_qdd(Multi_InputHandle, &Mouse_move, Flags, x, y)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Mouse_wheel
//

#define EventEnabledMouse_wheel() ((Multi_InputEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Mouse_wheel
//
#define EventWriteMouse_wheel(Flags, zDelta, x, y)\
        EventEnabledMouse_wheel() ?\
        Template_qddd(Multi_InputHandle, &Mouse_wheel, Flags, zDelta, x, y)\
        : ERROR_SUCCESS\

//
// Enablement check macro for Key_down
//

#define EventEnabledKey_down() ((Multi_InputEnableBits[0] & 0x00000001) != 0)

//
// Event Macro for Key_down
//
#define EventWriteKey_down(Virtual_key_code, Key_name, Repeat_count, Flags)\
        EventEnabledKey_down() ?\
        Template_qsqq(Multi_InputHandle, &Key_down, Virtual_key_code, Key_name, Repeat_count, Flags)\
        : ERROR_SUCCESS\

#endif // MCGEN_DISABLE_PROVIDER_CODE_GENERATION


//
// Allow Diasabling of code generation
//
#ifndef MCGEN_DISABLE_PROVIDER_CODE_GENERATION

//
// Template Functions 
//
//
//Template from manifest : T_Start
//
#ifndef Template_sd_def
#define Template_sd_def
ETW_INLINE
ULONG
Template_sd(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_opt_ LPCSTR  _Arg0,
    _In_ const signed int  _Arg1
    )
{
#define ARGUMENT_COUNT_sd 2

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_sd];

    EventDataDescCreate(&EventData[0], 
                        (_Arg0 != NULL) ? _Arg0 : "NULL",
                        (_Arg0 != NULL) ? (ULONG)((strlen(_Arg0) + 1) * sizeof(CHAR)) : (ULONG)sizeof("NULL"));

    EventDataDescCreate(&EventData[1], &_Arg1, sizeof(const signed int)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_sd, EventData);
}
#endif

//
//Template from manifest : T_End
//
#ifndef Template_sdf_def
#define Template_sdf_def
ETW_INLINE
ULONG
Template_sdf(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_opt_ LPCSTR  _Arg0,
    _In_ const signed int  _Arg1,
    _In_ const float  _Arg2
    )
{
#define ARGUMENT_COUNT_sdf 3

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_sdf];

    EventDataDescCreate(&EventData[0], 
                        (_Arg0 != NULL) ? _Arg0 : "NULL",
                        (_Arg0 != NULL) ? (ULONG)((strlen(_Arg0) + 1) * sizeof(CHAR)) : (ULONG)sizeof("NULL"));

    EventDataDescCreate(&EventData[1], &_Arg1, sizeof(const signed int)  );

    EventDataDescCreate(&EventData[2], &_Arg2, sizeof(const float)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_sdf, EventData);
}
#endif

//
//Template from manifest : T_Mark
//
#ifndef Template_s_def
#define Template_s_def
ETW_INLINE
ULONG
Template_s(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_opt_ LPCSTR  _Arg0
    )
{
#define ARGUMENT_COUNT_s 1

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_s];

    EventDataDescCreate(&EventData[0], 
                        (_Arg0 != NULL) ? _Arg0 : "NULL",
                        (_Arg0 != NULL) ? (ULONG)((strlen(_Arg0) + 1) * sizeof(CHAR)) : (ULONG)sizeof("NULL"));

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_s, EventData);
}
#endif

//
//Template from manifest : T_ThreadID
//
#ifndef Template_ds_def
#define Template_ds_def
ETW_INLINE
ULONG
Template_ds(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_ const signed int  _Arg0,
    _In_opt_ LPCSTR  _Arg1
    )
{
#define ARGUMENT_COUNT_ds 2

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_ds];

    EventDataDescCreate(&EventData[0], &_Arg0, sizeof(const signed int)  );

    EventDataDescCreate(&EventData[1], 
                        (_Arg1 != NULL) ? _Arg1 : "NULL",
                        (_Arg1 != NULL) ? (ULONG)((strlen(_Arg1) + 1) * sizeof(CHAR)) : (ULONG)sizeof("NULL"));

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_ds, EventData);
}
#endif

//
//Template from manifest : T_Mark2I
//
#ifndef Template_sdd_def
#define Template_sdd_def
ETW_INLINE
ULONG
Template_sdd(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_opt_ LPCSTR  _Arg0,
    _In_ const signed int  _Arg1,
    _In_ const signed int  _Arg2
    )
{
#define ARGUMENT_COUNT_sdd 3

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_sdd];

    EventDataDescCreate(&EventData[0], 
                        (_Arg0 != NULL) ? _Arg0 : "NULL",
                        (_Arg0 != NULL) ? (ULONG)((strlen(_Arg0) + 1) * sizeof(CHAR)) : (ULONG)sizeof("NULL"));

    EventDataDescCreate(&EventData[1], &_Arg1, sizeof(const signed int)  );

    EventDataDescCreate(&EventData[2], &_Arg2, sizeof(const signed int)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_sdd, EventData);
}
#endif

//
//Template from manifest : T_Mark1F
//
#ifndef Template_sf_def
#define Template_sf_def
ETW_INLINE
ULONG
Template_sf(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_opt_ LPCSTR  _Arg0,
    _In_ const float  _Arg1
    )
{
#define ARGUMENT_COUNT_sf 2

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_sf];

    EventDataDescCreate(&EventData[0], 
                        (_Arg0 != NULL) ? _Arg0 : "NULL",
                        (_Arg0 != NULL) ? (ULONG)((strlen(_Arg0) + 1) * sizeof(CHAR)) : (ULONG)sizeof("NULL"));

    EventDataDescCreate(&EventData[1], &_Arg1, sizeof(const float)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_sf, EventData);
}
#endif

//
//Template from manifest : T_Mark2F
//
#ifndef Template_sff_def
#define Template_sff_def
ETW_INLINE
ULONG
Template_sff(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_opt_ LPCSTR  _Arg0,
    _In_ const float  _Arg1,
    _In_ const float  _Arg2
    )
{
#define ARGUMENT_COUNT_sff 3

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_sff];

    EventDataDescCreate(&EventData[0], 
                        (_Arg0 != NULL) ? _Arg0 : "NULL",
                        (_Arg0 != NULL) ? (ULONG)((strlen(_Arg0) + 1) * sizeof(CHAR)) : (ULONG)sizeof("NULL"));

    EventDataDescCreate(&EventData[1], &_Arg1, sizeof(const float)  );

    EventDataDescCreate(&EventData[2], &_Arg2, sizeof(const float)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_sff, EventData);
}
#endif

//
//Template from manifest : T_FrameMark
//
#ifndef Template_df_def
#define Template_df_def
ETW_INLINE
ULONG
Template_df(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_ const signed int  _Arg0,
    _In_ const float  _Arg1
    )
{
#define ARGUMENT_COUNT_df 2

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_df];

    EventDataDescCreate(&EventData[0], &_Arg0, sizeof(const signed int)  );

    EventDataDescCreate(&EventData[1], &_Arg1, sizeof(const float)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_df, EventData);
}
#endif

//
//Template from manifest : T_MouseClick
//
#ifndef Template_dqdd_def
#define Template_dqdd_def
ETW_INLINE
ULONG
Template_dqdd(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_ const signed int  _Arg0,
    _In_ const unsigned int  _Arg1,
    _In_ const signed int  _Arg2,
    _In_ const signed int  _Arg3
    )
{
#define ARGUMENT_COUNT_dqdd 4

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_dqdd];

    EventDataDescCreate(&EventData[0], &_Arg0, sizeof(const signed int)  );

    EventDataDescCreate(&EventData[1], &_Arg1, sizeof(const unsigned int)  );

    EventDataDescCreate(&EventData[2], &_Arg2, sizeof(const signed int)  );

    EventDataDescCreate(&EventData[3], &_Arg3, sizeof(const signed int)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_dqdd, EventData);
}
#endif

//
//Template from manifest : T_MouseMove
//
#ifndef Template_qdd_def
#define Template_qdd_def
ETW_INLINE
ULONG
Template_qdd(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_ const unsigned int  _Arg0,
    _In_ const signed int  _Arg1,
    _In_ const signed int  _Arg2
    )
{
#define ARGUMENT_COUNT_qdd 3

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_qdd];

    EventDataDescCreate(&EventData[0], &_Arg0, sizeof(const unsigned int)  );

    EventDataDescCreate(&EventData[1], &_Arg1, sizeof(const signed int)  );

    EventDataDescCreate(&EventData[2], &_Arg2, sizeof(const signed int)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_qdd, EventData);
}
#endif

//
//Template from manifest : T_MouseWheel
//
#ifndef Template_qddd_def
#define Template_qddd_def
ETW_INLINE
ULONG
Template_qddd(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_ const unsigned int  _Arg0,
    _In_ const signed int  _Arg1,
    _In_ const signed int  _Arg2,
    _In_ const signed int  _Arg3
    )
{
#define ARGUMENT_COUNT_qddd 4

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_qddd];

    EventDataDescCreate(&EventData[0], &_Arg0, sizeof(const unsigned int)  );

    EventDataDescCreate(&EventData[1], &_Arg1, sizeof(const signed int)  );

    EventDataDescCreate(&EventData[2], &_Arg2, sizeof(const signed int)  );

    EventDataDescCreate(&EventData[3], &_Arg3, sizeof(const signed int)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_qddd, EventData);
}
#endif

//
//Template from manifest : T_KeyPress
//
#ifndef Template_qsqq_def
#define Template_qsqq_def
ETW_INLINE
ULONG
Template_qsqq(
    _In_ REGHANDLE RegHandle,
    _In_ PCEVENT_DESCRIPTOR Descriptor,
    _In_ const unsigned int  _Arg0,
    _In_opt_ LPCSTR  _Arg1,
    _In_ const unsigned int  _Arg2,
    _In_ const unsigned int  _Arg3
    )
{
#define ARGUMENT_COUNT_qsqq 4

    EVENT_DATA_DESCRIPTOR EventData[ARGUMENT_COUNT_qsqq];

    EventDataDescCreate(&EventData[0], &_Arg0, sizeof(const unsigned int)  );

    EventDataDescCreate(&EventData[1], 
                        (_Arg1 != NULL) ? _Arg1 : "NULL",
                        (_Arg1 != NULL) ? (ULONG)((strlen(_Arg1) + 1) * sizeof(CHAR)) : (ULONG)sizeof("NULL"));

    EventDataDescCreate(&EventData[2], &_Arg2, sizeof(const unsigned int)  );

    EventDataDescCreate(&EventData[3], &_Arg3, sizeof(const unsigned int)  );

    return EventWrite(RegHandle, Descriptor, ARGUMENT_COUNT_qsqq, EventData);
}
#endif

#endif // MCGEN_DISABLE_PROVIDER_CODE_GENERATION

#if defined(__cplusplus)
};
#endif

